@@ -4,6 +4,7 @@ import numpy as np
 import re
 import os
 import sys
+import pickle
 from pathlib import Path
 from pdb import set_trace
 from cvnn.utils import create_folder
@@ -824,13 +825,13 @@ class MonteCarloPlotter(Plotter):
                     data_50.append(stats['50%'])
                     data_25.append(stats['25%'])
                     data_75.append(stats['75%'])
-            ax.plot(x, data_mean, color=DEFAULT_MATPLOTLIB_COLORS[i],
+            ax.plot(x, data_mean, color=DEFAULT_MATPLOTLIB_COLORS[i%len(DEFAULT_MATPLOTLIB_COLORS)],
                     label=label.replace('_', ' ') + ' mean')
-            ax.plot(x, data_50, '--', color=DEFAULT_MATPLOTLIB_COLORS[i],
+            ax.plot(x, data_50, '--', color=DEFAULT_MATPLOTLIB_COLORS[i%len(DEFAULT_MATPLOTLIB_COLORS)],
                     label=label.replace('_', ' ') + ' median')
-            ax.fill_between(x, data_25, data_75, color=DEFAULT_MATPLOTLIB_COLORS[i], alpha=.4,
+            ax.fill_between(x, data_25, data_75, color=DEFAULT_MATPLOTLIB_COLORS[i%len(DEFAULT_MATPLOTLIB_COLORS)], alpha=.4,
                             label=label.replace('_', ' ') + ' interquartile')
-            ax.fill_between(x, data_min, data_max, color=DEFAULT_MATPLOTLIB_COLORS[i], alpha=.15,
+            ax.fill_between(x, data_min, data_max, color=DEFAULT_MATPLOTLIB_COLORS[i%len(DEFAULT_MATPLOTLIB_COLORS)], alpha=.15,
                             label=label.replace('_', ' ') + ' border')
         for label in sorted(self.pandas_dict.keys()):
             if network_filter is not None and label not in network_filter:
@@ -899,8 +900,8 @@ class MonteCarloPlotter(Plotter):
                     x=x + x_rev,
                     y=data_max + data_min,
                     fill='toself',
-                    fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0.1),
-                    line_color=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0),
+                    fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], 0.1),
+                    line_color=add_transparency(DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], 0),
                     showlegend=True,
                     name=label.replace('_', ' ') + " borders",
                 ))
@@ -908,19 +909,19 @@ class MonteCarloPlotter(Plotter):
                 x=x + x_rev,
                 y=data_75 + data_25,
                 fill='toself',
-                fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0.2),
-                line_color=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0),
+                fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], 0.2),
+                line_color=add_transparency(DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], 0),
                 showlegend=True,
                 name=label.replace('_', ' ') + " interquartile",
             ))
             fig.add_trace(go.Scatter(
                 x=x, y=data_mean,
-                line_color=DEFAULT_PLOTLY_COLORS[i],
+                line_color=DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)],
                 name=label.replace('_', ' ') + " mean",
             ))
             fig.add_trace(go.Scatter(
                 x=x, y=data_50,
-                line=dict(color=DEFAULT_PLOTLY_COLORS[i], dash='dash'),
+                line=dict(color=DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], dash='dash'),
                 name=label.replace('_', ' ') + " median",
             ))
         for label in sorted(self.pandas_dict.keys()):
@@ -955,7 +956,7 @@ class MonteCarloPlotter(Plotter):
             data_mean_test = data[data['stats'] == filter_label]["val_" + key].tolist()
             fig.add_trace(go.Scatter(
                 x=x, y=data_mean_test,
-                line_color=DEFAULT_PLOTLY_COLORS[i],
+                line_color=DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)],
                 name=label + " test",
             ))
             data_mean_train = data[data['stats'] == filter_label][key].tolist()
@@ -985,7 +986,7 @@ class MonteCarloAnalyzer:
     This class works with the run_data.csv generated with cvnn.montecarlo.MonteCarlo class
     """
 
-    def __init__(self, df=None, path=None):
+    def __init__(self, df=None, path=None, history_dictionary: Optional[dict] = None):
         """
         There are 2 ways to use this class:
             1. Either give the data as a pandas DataFrame.
@@ -1002,6 +1003,8 @@ class MonteCarloAnalyzer:
                     This is useful when you want to plot together different MonteCarlo.run() results.
                     This enables to run two simulations of 50 iterations each and plot them as if
                         it was a single run of 100 iterations.
+        :param history_dictionary: Optional dictionary. This parameter is only used if df and path are None.
+            Dictionary with the models as keys and a list of full paths to the model history pickle file.
         """
         self.confusion_matrix = []
         if path is not None and df is not None:  # I have data and the place where I want to save it
@@ -1009,7 +1012,7 @@ class MonteCarloAnalyzer:
             self.path = Path(path)
             self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
         elif path is not None and df is None:  # Load df from Path
-            if not path.endswith("run_data.csv") or not path.endswith("run_data"): 
+            if not (path.endswith("run_data.csv") or path.endswith("run_data")):
                 self.df = pd.DataFrame()
                 data_files = get_data_files_list(path)
                 for file in data_files:
@@ -1025,17 +1028,41 @@ class MonteCarloAnalyzer:
                     path += '.csv'
                 self.df = pd.read_csv(Path(path), index_col=False)  # Path(__file__).parents[1].absolute() /
                 self.path = Path(os.path.split(path)[0])  # Keep only the path and not the filename
+            self.df["network"] = self.df["network"].astype(str)  # Make sure networks name are string
         elif path is None and df is not None:  # Save df into default path
             self.path = create_folder("./log/montecarlo/")
             self.df = df  # DataFrame with all the data
             self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
         else:  # I have nothing
-            self.path = create_folder("./log/montecarlo/")
-            self.df = pd.DataFrame()
+            if history_dictionary is not None:
+                self.df = self._open_dict(history_dictionary)
+                self.path = create_folder("./log/montecarlo/")
+                self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
+                self.save_stat_results()       # Create the stat results for the plotter
+            else:
+                self.df = pd.DataFrame()
+                self.path = create_folder("./log/montecarlo/")
+                self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
+
+            # set_trace()
         self.plotable_info = ['loss', 'val_loss', 'accuracy', 'val_accuracy']  # TODO: Consider delete
         self.monte_carlo_plotter = MonteCarloPlotter(self.path)
         self.summary = []
 
+    @staticmethod
+    def _open_dict(history_dictionary: dict):
+        assert isinstance(history_dictionary, dict), f"history_dictionary must be a dictionary"
+        df = pd.DataFrame()
+        for model in history_dictionary.keys():
+            for path in history_dictionary[model]:
+                with open(path, 'rb') as f:
+                    saved_history = pickle.load(f)
+                tmp_df = pd.DataFrame(saved_history)
+                tmp_df['epoch'] = list(range(0, len(saved_history['accuracy'])))
+                tmp_df['network'] = model
+                df = pd.concat([df, tmp_df])
+        return df
+
     def set_df(self, df, conf_mat=None):
         self.df = df  # DataFrame with all the data
         self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
@@ -1091,7 +1118,7 @@ class MonteCarloAnalyzer:
                     self.box_plot(key=key, extension=extension, library=lib, showfig=showfig, savefig=savefig,
                                   network_filter=network_filter)
                 except:
-                    logger.warning("Could not plot " + key + " Histogram with " + str(lib), exc_info=True)
+                    logger.warning("Could not plot " + key + " Box Plot with " + str(lib), exc_info=True)
                 try:
                     self.plot_histogram(key=key, library=lib, showfig=showfig, savefig=savefig, extension=extension)
                 except np.linalg.LinAlgError:
@@ -1106,7 +1133,8 @@ class MonteCarloAnalyzer:
                     logger.warning(f"Could not plot {key} line_confidence_interval with {lib}", exc_info=True)
 
     def box_plot(self, epoch: int = -1, library: str = 'plotly', key: str = 'val_accuracy',
-                 showfig: bool = False, savefig: bool = True, extension: str = '.svg', network_filter=None):
+                 showfig: bool = False, savefig: bool = True, extension: str = '.svg', network_filter=None,
+                 early_stop: bool = True):
         """
         Saves/shows a box plot of the results.
         :param epoch: Which epoch to use for the box plot. If -1 (default) it will use the last epoch.
@@ -1118,37 +1146,49 @@ class MonteCarloAnalyzer:
         :param savefig: If True, it saves the figure at: self.path / "plots/box_plot/"
         :param extension: file extensions (default svg) to be used when saving the file
             (only used when library is seaborn).
+        :param early_stop: Default True. Compatible mode if using early stop.
+            This takes precedence over epoch param.
+            Recommended to always use it at True unless you want a particular epoch.
+            Uses the last epoch of each iteration as they might differ when using early stopping.
         """
         if library == 'plotly':
-            self._box_plot_plotly(key=key, epoch=epoch, showfig=showfig, savefig=savefig)
+            self._box_plot_plotly(key=key, epoch=epoch, showfig=showfig, savefig=savefig, early_stop=early_stop)
         elif library == 'seaborn':
             self._box_plot_seaborn(key=key, epoch=epoch, showfig=showfig, savefig=savefig, extension=extension,
-                                   network_filter=network_filter)
+                                   network_filter=network_filter, early_stop=early_stop)
         else:
             logger.warning("Warning: Unrecognized library to plot " + library)
             return None
 
-    def _box_plot_plotly(self, epoch=-1, key='val_accuracy', showfig=False, savefig=True):
+    def _box_plot_plotly(self, epoch=-1, key='val_accuracy', showfig=False, savefig=True, early_stop=True):
         if 'plotly' not in AVAILABLE_LIBRARIES:
             logger.warning("No Plotly installed, function " + self._box_plot_plotly.__name__ +
                            " was called but will be omitted")
             return None
         fig = go.Figure()
-        if epoch == -1:
-            epoch = max(self.df.epoch)
         networks_available = self.df.network.unique()
-        for col, net in enumerate(sorted(networks_available)):
-            filter = [a == net and b == epoch for a, b in zip(self.df.network, self.df.epoch)]
+        # Take the corresponging epoch
+        if early_stop:  # Take the last epoch of each iteration
+            # Next lines searches for epoch reset (meaning I got to the last epoch)
+            filter = [self.df.iloc[i]['epoch'] > self.df.iloc[i + 1]['epoch'] for i in range(0, len(self.df.index) - 1)]
+            filter = filter + [True]    # The last element is an epoch that should be taken
             data = self.df[filter]
+        else:
+            if epoch == -1:
+                epoch = max(self.df.epoch)
+            filter = self.df['epoch'] == epoch
+            data = self.df[filter]
+        for col, net in enumerate(sorted(networks_available)):
+            net_data = data[data["network"]==net]
             fig.add_trace(go.Box(
-                y=data[key],
+                y=net_data[key],
                 # x=[self.x[i]] * len(data[key]),
                 name=net.replace('_', ' '),
                 whiskerwidth=0.2,
                 notched=True,  # confidence intervals for the median
-                fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[col], 0.5),
+                fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[col%len(DEFAULT_PLOTLY_COLORS)], 0.5),
                 boxpoints='suspectedoutliers',  # to mark the suspected outliers
-                line=dict(color=DEFAULT_PLOTLY_COLORS[col]),
+                line=dict(color=DEFAULT_PLOTLY_COLORS[col%len(DEFAULT_PLOTLY_COLORS)]),
                 boxmean=True  # Interesting how sometimes it falls outside the box
             ))
         fig.update_layout(
@@ -1181,16 +1221,20 @@ class MonteCarloAnalyzer:
             plt.close(fig)
 
     def _box_plot_seaborn(self, epoch=-1, key='val_accuracy', showfig=False, savefig=True, extension='.svg',
-                          network_filter=None):
+                          network_filter=None, early_stop=True):
         if 'seaborn' not in AVAILABLE_LIBRARIES:
             logger.warning("No Seaborn installed, function " + self._box_plot_seaborn.__name__ +
                            " was called but will be omitted")
             return None
-        if epoch == -1:
-            epoch = max(self.df.epoch)
         # Prepare data
-        filter = self.df['epoch'] == epoch
-        data = self.df[filter]
+        if early_stop:
+            filter = [self.df.iloc[i]['epoch'] > self.df.iloc[i + 1]['epoch'] for i in range(0, len(self.df.index) - 1)] + [True]
+            data = self.df[filter]
+        else:
+            if epoch == -1:
+                epoch = max(self.df.epoch)
+            filter = self.df['epoch'] == epoch
+            data = self.df[filter]
         if network_filter:
             filter = data['network'].isin(network_filter)
             data = data[filter]
@@ -1269,15 +1313,15 @@ class MonteCarloAnalyzer:
 
                 fig.add_traces(go.Scatter3d(x=[epoch] * len(counts), y=bins, z=counts,
                                             mode='lines', name=net.replace("_", " ") + "; epoch: " + str(epoch),
-                                            surfacecolor=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0),
+                                            surfacecolor=add_transparency(DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], 0),
                                             # surfaceaxis=0,
-                                            line=dict(color=DEFAULT_PLOTLY_COLORS[i], width=4)
+                                            line=dict(color=DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], width=4)
                                             )
                                )
                 verts, tri = triangulate_histogram([epoch] * len(counts), bins, counts)
                 x, y, z = verts.T
                 I, J, K = tri.T
-                fig.add_traces(go.Mesh3d(x=x, y=y, z=z, i=I, j=J, k=K, color=DEFAULT_PLOTLY_COLORS[i], opacity=0.4))
+                fig.add_traces(go.Mesh3d(x=x, y=y, z=z, i=I, j=J, k=K, color=DEFAULT_PLOTLY_COLORS[i%len(DEFAULT_PLOTLY_COLORS)], opacity=0.4))
         for net in networks_availables:
             title += net + ' '
         title += key + " comparison"
@@ -1294,7 +1338,8 @@ class MonteCarloAnalyzer:
                             config=PLOTLY_CONFIG, auto_open=False)
 
     def plot_histogram(self, key: str = 'val_accuracy', epoch: int = -1, library: str = 'seaborn',
-                       showfig: bool = False, savefig: bool = True, title: str = '', extension: str = ".svg"):
+                       showfig: bool = False, savefig: bool = True, title: str = '', extension: str = ".svg",
+                       early_stop: bool = True):
         """
         Saves/shows a histogram of the results.
 
@@ -1308,10 +1353,14 @@ class MonteCarloAnalyzer:
         :param savefig: If True, it saves the figure at: self.path / "plots/box_plot/"
         :param title: Figure title
         :param extension: file extensions (default svg) to be used when saving the file (ignored if library is plotly).
+        :param early_stop: Default True. Compatible mode if using early stop.
+            This takes precedence over epoch param.
+            Recommended to always use it at True unless you want a particular epoch.
+            Uses the last epoch of each iteration as they might differ when using early stopping.
         """
         if library == 'matplotlib':
             self._plot_histogram_matplotlib(key=key, epoch=epoch, showfig=showfig, savefig=savefig, title=title,
-                                            extension=extension)
+                                            extension=extension, early_stop=early_stop)
         elif library == 'plotly':
             self._plot_histogram_plotly(key=key, epoch=epoch, showfig=showfig, savefig=savefig, title=title)
         elif library == 'seaborn':
@@ -1322,7 +1371,7 @@ class MonteCarloAnalyzer:
             return None
 
     def _plot_histogram_matplotlib(self, key='val_accuracy', epoch=-1,
-                                   showfig=False, savefig=True, title='', extension=".svg"):
+                                   showfig=False, savefig=True, title='', extension=".svg", early_stop: bool = True):
         if 'matplotlib' not in AVAILABLE_LIBRARIES:
             logger.warning("No Matplotlib installed, function " + self._plot_histogram_matplotlib.__name__ +
                            " was called but will be omitted")
@@ -1333,11 +1382,18 @@ class MonteCarloAnalyzer:
         min_ax = 1.0
         max_ax = 0.0
         networks_availables = self.df.network.unique()
-        if epoch == -1:
-            epoch = max(self.df.epoch)
+        if early_stop:  # Take the last epoch of each iteration
+            # Next lines searches for epoch reset (meaning I got to the last epoch)
+            filter = [self.df.iloc[i]['epoch'] > self.df.iloc[i + 1]['epoch'] for i in range(0, len(self.df.index) - 1)]
+            filter = filter + [True]  # The last element is an epoch that should be taken
+            df_pre_filtered = self.df[filter]
+        else:
+            if epoch == -1:
+                epoch = max(self.df.epoch)
+            filter = self.df['epoch'] == epoch
+            df_pre_filtered = self.df[filter]
         for net in networks_availables:
-            filter = [a == net and b == epoch for a, b in zip(self.df.network, self.df.epoch)]
-            data = self.df[filter]  # Get only the data to plot
+            data = df_pre_filtered[df_pre_filtered["network"]==net]  # Get only the data to plot
             ax.hist(data[key], bins, alpha=0.5, label=net.replace("_", " "))
             min_ax = min(min_ax, min(data[key]))
             max_ax = max(max_ax, max(data[key]))
@@ -1353,20 +1409,28 @@ class MonteCarloAnalyzer:
             plt.close(fig)
         return fig, ax
 
-    def _plot_histogram_plotly(self, key='val_accuracy', epoch=-1, showfig=False, savefig=True, title=''):
+    def _plot_histogram_plotly(self, key='val_accuracy', epoch=-1, showfig=False, savefig=True, title='',
+                               early_stop: bool = True):
         if 'plotly' not in AVAILABLE_LIBRARIES:
             logger.warning("No Plotly installed, function " + self._plot_histogram_plotly.__name__ +
                            " was called but will be omitted")
             return None
         networks_availables = self.df.network.unique()
-        if epoch == -1:
-            epoch = max(self.df.epoch)
+        if early_stop:  # Take the last epoch of each iteration
+            # Next lines searches for epoch reset (meaning I got to the last epoch)
+            filter = [self.df.iloc[i]['epoch'] > self.df.iloc[i + 1]['epoch'] for i in range(0, len(self.df.index) - 1)]
+            filter = filter + [True]  # The last element is an epoch that should be taken
+            df_pre_filtered = self.df[filter]
+        else:
+            if epoch == -1:
+                epoch = max(self.df.epoch)
+            filter = self.df['epoch'] == epoch
+            df_pre_filtered = self.df[filter]
         hist_data = []
         group_labels = []
         for net in networks_availables:
             title += net + ' '
-            filter = [a == net and b == epoch for a, b in zip(self.df.network, self.df.epoch)]
-            data = self.df[filter]  # Get only the data to plot
+            data = df_pre_filtered[df_pre_filtered["network"]==net]  # Get only the data to plot
             hist_data.append(data[key].to_list())
             group_labels.append(net.replace("_", " "))
             # fig.add_trace(px.histogram(np.array(data[key]), marginal="box"))
@@ -1395,7 +1459,7 @@ class MonteCarloAnalyzer:
         return fig
 
     def _plot_histogram_seaborn(self, key='val_accuracy', epoch=-1,
-                                showfig=True, savefig=True, title='', extension=".svg"):
+                                showfig=True, savefig=True, title='', extension=".svg", early_stop: bool = True):
         if 'seaborn' not in AVAILABLE_LIBRARIES:
             logger.warning("No Seaborn installed, function " + self._plot_histogram_seaborn.__name__ +
                            " was called but will be omitted")
@@ -1407,10 +1471,15 @@ class MonteCarloAnalyzer:
         max_ax = 0.0
         # ax = None
         networks_availables = self.df.network.unique()
-        if epoch == -1:
-            epoch = max(self.df.epoch)
-        filter = [e == epoch for e in self.df.epoch]
-        data = self.df[filter]
+        if early_stop:
+            filter = [self.df.iloc[i]['epoch'] > self.df.iloc[i + 1]['epoch'] for i in
+                      range(0, len(self.df.index) - 1)] + [True]
+            data = self.df[filter]
+        else:
+            if epoch == -1:
+                epoch = max(self.df.epoch)
+            filter = self.df['epoch'] == epoch
+            data = self.df[filter]
         ax = sns.histplot(data, x=key, hue='network', bins=bins, legend=True)
         min_ax = min(min_ax, min(data[key]))
         max_ax = max(max_ax, max(data[key]))
@@ -1468,6 +1537,6 @@ if __name__ == "__main__":
     """
 
 __author__ = 'J. Agustin BARRACHINA'
-__version__ = '0.1.49'
+__version__ = '0.1.51'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
